let socket = io()
let serial_data

let pi2 = Math.PI * 2
//10-bit value that can come in from the phototransistor
const max_value = 1023

//Sizes
const width = 800
const height = 800

//Load the font for usage in the canvas
const font_name = 'Source Code Pro'
document.fonts.load(`normal 900 10px "${font_name}"`)

//Values for the behavior of the circles
const colors = ["#22226d","#66489f", "#ff33a5", "#efb605"].reverse()
const radiusRanges = [3, 20]
const forceRanges = [0.5, 5]

let particles = []
let angle = 0
let radius = width/2 * 0.7

////////////////////// Create Scales //////////////////////
//Number of new particles based on value
const scale_particles = d3.scalePow()
    .exponent(3)
    .domain([400, max_value])
    .range([2, 10])
    .clamp(true)

//The "delta angle" that gets added on each new call to the draw function
const scale_angle = d3.scalePow()
    .exponent(3)
    .domain([0, max_value])
    .range([0.01, 0.05])

//Color of the background
const scale_color = d3.scaleLinear()
    .domain(d3.range(colors.length).map(d => { return d/(colors.length-1) * max_value}))
    .range(colors)
    .clamp(true)

//Radius long which the particles are running
const scale_radius = d3.scaleLinear()
    .domain([0, max_value])
    .range([width/2 * 0.4, width/2 * 0.7])
    .clamp(true)

/////////////////// Set-up the Container //////////////////

//Set-up the canvas
const canvas = document.createElement("canvas")
const ctx = canvas.getContext("2d")

//Update the canvas size - taking retina devices into account
const pixel_ratio = 2 //for retina devices
canvas.width = width * pixel_ratio
canvas.height = height * pixel_ratio
canvas.style.width = `${width}px`
canvas.style.height = `${height}px`
ctx.scale(pixel_ratio, pixel_ratio)

//Move the origin to the center
//This is easier when working with a circular layout
ctx.translate(width/2, height/2)

//Add the canvas to the "chart" div
document.getElementById("chart").appendChild(canvas)

////////////////// Text on the background /////////////////
//Set the font - only once, since we're never changing it
ctx.font = `900 normal 180px ${font_name}`
ctx.textBaseline = "middle"
ctx.textAlign = "center"

function updateText(data) {
    ctx.fillStyle = "white"
    ctx.fillText(data, 0, 0)
}//function updateText

//////////////////////// Particles ////////////////////////
function updateParticle(data) { 
    ctx.globalCompositeOperation = "screen"

    //Create new particles
    angle += scale_angle(data)
    radius = scale_radius(data)
    for (let j = 0; j < scale_particles(data); j++) 
        spawn(radius * Math.cos(angle), radius * Math.sin(angle))

    //Remove non-alive particles
    particles = particles.filter(d => d.alive)

    particles.forEach(d => {
        //Move each particle
        move(d)
        //Draw each particle
        ctx.fillStyle = d.color
        ctx.beginPath()
        ctx.arc(d.x, d.y, d.radius, 0, pi2)
        ctx.fill()
    })//forEach
}//function updateParticle

/////////////////////// Full Redraw ///////////////////////
function draw() {
    if(serial_data) { //Only draw if serial_data is known
        //Update the background
        ctx.globalCompositeOperation = "source-over"
        ctx.fillStyle = scale_color(serial_data)
        ctx.fillRect(-width/2, -height/2, width, height)

        updateText(serial_data)
        updateParticle(serial_data)
    }//if
}//function draw

///////////////////////////////////////////////////////////
/////// Functions to create and move the particles ////////
///////////////////////////////////////////////////////////

//Code heavily based on http://codepen.io/soulwire/pen/foktm
//Calculates new position
function move(d) {
    d.x += d.vx
    d.y += d.vy

    d.vx *= d.drag
    d.vy *= d.drag

    d.theta += getRandomNumber( -0.5, 0.5 ) * d.wander
    d.vx += Math.sin( d.theta ) * 0.5
    d.vy += Math.cos( d.theta ) * 0.5

    d.radius *= d.age
    d.alive = d.radius > 0.5
}//move

//Create a particle
function spawn ( x, y ) {
    
    //Play around with these numbers to get different effects
    let particle = {
        x: x,
        y: y,
        alive: true,
        radius: getRandomNumber( radiusRanges[0], radiusRanges[1] ),
        wander: getRandomNumber( 1, 1.5 ),
        color: colors[ Math.round( getRandomNumber(0, colors.length-1)) ],
        drag: getRandomNumber( 0.2, 0.99 ),
        age: getRandomNumber( 0.92, 0.98 ),
        theta: getRandomNumber( 0,  2 * Math.PI ),
        force: getRandomNumber( forceRanges[0], forceRanges[1] )
    }

    particle.vx = Math.sin( particle.theta ) * particle.force
    particle.vy = Math.cos( particle.theta ) * particle.force

    particles.push( particle )
}//spawn

function getRandomNumber(start, end) { return ((Math.random() * (end-start)) + start) }

///////////////////////// Get Data ////////////////////////
//Get the data from the serial port and update the visual
socket.on('connect', function () {
    setInterval(draw, 20)
    socket.on('parsed-data', function (data) {
        //console.log(data)
        serial_data = data
    })//socket.on
})//socket.on

